###################################
# PROLOG SECTION
# hangman.py
# Program to play the traditional game of hangman.
# The computer picks a word and the player
# has to guess it before making 7 wrong
# guesses of letters in the word.
# Each time the user makes an incorrect guess
# of a letter, one more body part gets added
# to the text graphic.
#
# Tested with Python 3.7.9 on Windows 10
# Fri July 16, 2021
#
# Copyright ©. All rights reserved. NAF 2021.
#
# Possible future enhancements:
# Unresolved bugs:
###################################

###################################
# ENVIRONMENT SETUP SECTION
# Imports: random
###################################

# import random number routines
from random import *

###################################
# FUNCTION DEFINITION SECTION
# Definitions:
#   print_game_rules()
#   prompt_for_letter()
#   display_figure()
###################################

#
# Function to print out the rules of the game.
#
# The function also gives a clue as to what kind of
# word it is and also tells how many letters it has in it.
#
def print_game_rules(max_incorrect,word_len):
    print("Welcome to the hangman game!")
    print()
    print("You get",max_incorrect,"tries to guess letters in the mystery word.")
    print("Think of letters that might be in the word and type them on your")
    print("keyboard. If you make",max_incorrect,"incorrect letter guesses, the")
    print("picture will show that you are hanged and you lose the game. If you guess")
    print("all the letters in the word before that, you win the game.")
    print()
    # print the category and length of the mystery word
    print("The word is an animal, and it has",word_len,"letters in it.")

#
# Function to display a text graphics picture
# of the hanging of the player.
#
# The "bad_guesses" parameter tells how many incorrect letters
# have been guessed in the player's attempt to guess the
# letters in the randomly chosen mystery word. Once
# 7 incorrect guesses have been made, the player is
# hanged and the game is over.
#
# The pictures are a tuple of triple-quoted strings
# containing text mode drawings.
#
def display_figure(bad_guesses):
    figures=(
# drawn and redrawn if no incorrect guesses have been made
# 0 incorrect letters guessed.
"""
   +------+
   |
   |
   |
   |   
   | 
   |
================
""",
# 1 bad guess
"""
   +------+
   |      O
   | 
   |
   |   
   | 
   |
================
""",
# 2 bad guesses
"""
   +------+
   |      O
   |      +
   |
   |   
   | 
   |
================
""",
# 3 bad guesses
"""
   +------+
   |      O
   |   ---+
   |
   |   
   | 
   |
================
""",
# 4 bad guesses
"""
   +------+
   |      O
   |   ---+---
   |
   |   
   | 
   |
================
""",
# 5 bad guesses
"""
   +------+
   |      O
   |   ---+---
   |      |
   |   
   | 
   |
================
""",
# 6 bad guesses
"""
   +------+
   |      O
   |   ---+---
   |      |
   |     /   
   |    / 
   |
================
""",
# 7 bad guesses (game over)
"""
   +------+
   |      O
   |   ---+---
   |      |
   |     / \    
   |    /   \ 
   |
================
""")
    
    # Print out the text graphic figure based on the number of bad letter guesses.
    print(figures[bad_guesses])
    print()
                   
# Function to prompt the player to guess a letter in the word.
#
# The letter guess is read into a string. Any leading or trailing
# blanks are removed from the string and the result is converted
# to lowercase. The function returns the resulting string to
# the main program. 
#
def prompt_for_letter():
    print()
    # Assign the player's guess to the "answer" string variable.
    answer = str(input("Guess a letter in the word: "))
    # Remove any leading or trailing blanks from the guess.
    answer = answer.strip()
    # Convert the guess to all lowercase letters.
    answer = answer.lower()
    print()
    # Return what the player typed after stripping and converting to lowercase.
    return answer
   

###################################
# PROCESSING INITIALIZATION SECTION
###################################

#
# Define the tuple of words to be guessed. One of these words
# is chosen at random.
#
# (all words are animals)
# (tuples can't be changed)
#
animal_words = ("horse","cow","dog","cat","hyena","antelope","monkey","ox",\
               "moose","woodchuck","mouse","zebra")

# Select a random word from the animal_words list.
# choice() is a random function from the random module.
theword = choice(animal_words)

# Assign the length of the word chosen to the variable "word_len".
word_len = len(theword)

# Create a list to display letters not yet guessed and also
# those that have been guessed. There is one entry in this
# list for each letter in the mystery word. The list will contain
# either "_" if the letter has not yet been guessed, or a letter
# if it has been guessed. So "__a_t" would mean the letters "a" and "t"
# have been correctly guessed and that there are still three letters that
# have not yet been guessed by the player.
guesses = word_len*['_']

#
# Define variables used to keep track of the state
# of the game.
#

# Set the maximum number of incorrect letter guesses allowed.
max_incorrect = 7

# Define a string containing the letters already guessed.
# There are none to begin with.
letters_tried = ""

# Initialize count of letters guessed correctly.
letters_correct=0

# Initialize the count of incorrect letter guesses made so far.
incorrect_guesses=0

###################################
# PROCESSING SECTION
# Loop asking the player for a letter guess
# until they guess the word or are hanged for too
# many wrong guesses.
# Branching code: if/else
# Looping code: while-loop, for-loop
###################################

# Print the game rules and a clue about the mystery word to be guessed
# by the player.
print_game_rules(max_incorrect, word_len)

# The main loop to ask the player for letter guesses and to display the hangman figure.
# The game ends when either the player guesses all the letters in the
# word or they get hanged for having too many bad guesses.

while (incorrect_guesses != max_incorrect) and (letters_correct != word_len):
    # Prompt the player to guess a letter in the word. Store the
    # guess in "letter".
    letter = prompt_for_letter()
    # Test that they guessed a single letter in the alphabet.
    if len(letter) == 1 and letter.isalpha():
        # The guess is a letter in the alphabet, did they already pick it?
        # We remember each letter that has been guessed. If they guess the
        # same letter a second time we reprompt them to guess another letter.
        #
        # Test to see if the letter just guessed has already been picked by
        # checking if it is in the string of previously guessed letters.
        if letters_tried.find(letter) >= 0:
            # Tell them they already picked this letter.
            print("You already picked",letter)
        else:
            # Add the latest letter to the end of the string that contains
            # all the letters guessed so far.
            letters_tried = letters_tried + letter
            # Check to see if the letter picked is in the word
            # by searching for it using the find() string method.
            # Store the result in "first_index". If -1 is returned by find(),
            # the letter guessed is not in the mystery word.
            first_index=theword.find(letter)
            if first_index == -1:
                # The letter just guessed is not in the mystery word!
                print("The letter '"+letter+"' is not in the word.")
                # Add 1 to the number of incorrect letters guessed.
                incorrect_guesses = incorrect_guesses+1
            else:
                # If we come here, the letter just guessed is in the
                # mystery word. It may even be in the word in more than
                # one position. Now update the "guesses" list to show
                # all the places in the mystery word that match the
                # letter that was just guessed.
                print("The letter '"+letter+"' is in the word.")
                # Loop through all the letters in the mystery word.
                for i in range(word_len):
                    # Test to see if the letter just guessed matches the
                    # letter in this position in the mystery word.
                    if letter == theword[i]:
                        # We have a match. Replace '_' in "guesses" with
                        # the letter just guessed.
                        guesses[i]=letter
                        # Add 1 to the number of letters correctly guessed.
                        letters_correct=letters_correct+1
                       
    else:
        # If we come here, the player didn't type in a letter between a to z
        # at the prompt.
        print("Please guess a single letter in the alphabet.")

    # Display the hangman figure for the number of incorrect guesses the player
    # has made so far.
    display_figure(incorrect_guesses)
    # Display the string of letters already tried.
    print("letters tried so far: ", letters_tried)
    # Display the current state of the "guesses" list showing
    # "_" for letters not yet guessed and a letter of the alphabet
    # for letters that have been guessed correctly.
    # The statements below will convert the items in the "guesses"
    # list into a single string and print it.
    guesses_str = ""
    for g in guesses:
        guesses_str = guesses_str+g
    print(guesses_str)

    # After the current guess, check for conditions that the game is about to end.
    
    # See if the game is over because they have reached the maximum number of
    # incorrect guess allowed. This means they are "hanged".
    if incorrect_guesses == max_incorrect:
        print("Sorry, too many incorrect guesses. You are hanged.")
        # Tell them what the mystery word was that they didn't guess.
        print("The word was '"+theword+"'.")
    
    # See if the user has guessed all the letters in the word, by checking that
    # the number of correct letter guesses is the same as the length of the
    # mystery word.
    if letters_correct==word_len:
       # Congratulate the player.
       print("You guessed all the letters in the word!")
       # Print out the mystery word.
       print("The word was '"+theword+"'.") 

###################################   
# CLEANUP, TERMINATION, AND EXIT SECTION
###################################

print("The game is over.")
