###################################
# PROLOG SECTION
# hangman.py
# Plays a text mode game of hangman
# where the computer picks a word and
# the player has to guess it before they
# have made 7 wrong guesses of letters in the
# word. Each time they make an incorrect guess of
# a letter, one more body part gets added to their
# picture on a scaffold.
#
# Tested with Python 3.7.9 on Windows 10
# Fri July 16, 2021
#
# Copyright ©. All rights reserved. NAF 2021.
#
# Possible future enhancements:
# Unresolved bugs:
###################################

###################################
# ENVIRONMENT SETUP SECTION
# Imports: random
###################################

# import random number routines
from random import *

###################################
# FUNCTION DEFINITION SECTION
# Definitions:
#   print_game_rules()
#   prompt_for_letter()
#   display_figure()
###################################

#
# Function to print out the rules of the game.
#
# The function also gives a clue as to what kind of
# word it is and also tells how many letters it has in it.
#
def print_game_rules(max_incorrect,word_len):
    print("Welcome to the hangman game!")
    print()
    print("You get",max_incorrect,"tries to guess letters in the mystery word.")
    print("Think of letters that might be in the word and type them on your")
    print("keyboard. If you make",max_incorrect,"incorrect letter guesses, the")
    print("picture will show that you are hanged and you lose the game. If you guess")
    print("all the letters in the word before that, you win the game.")
    print()
    # print the category and length of the mystery word
    print("The word is an animal, and it has",word_len,"letters in it.")

#
# Function to display a text graphics picture
# of the hanging of the player.
#
# The "bad_guesses" parameter tells how many incorrect letters
# have been guessed in the player's attempt to guess the
# letters in the randomly chosen mystery word. Once
# 7 incorrect guesses have been made, the player is
# hanged and the game is over.
#
# The pictures are a tuple of triple-quoted strings
# containing text mode drawings.
#
def display_figure(bad_guesses):
    figures=(
# drawn and redrawn if no incorrect guesses have been made
# 0 incorrect letters guessed.
"""
   +------+
   |
   |
   |
   |   
   | 
   |
================
""",
# 1 bad guess
"""
   +------+
   |      O
   | 
   |
   |   
   | 
   |
================
""",
# 2 bad guesses
"""
   +------+
   |      O
   |      +
   |
   |   
   | 
   |
================
""",
# 3 bad guesses
"""
   +------+
   |      O
   |   ---+
   |
   |   
   | 
   |
================
""",
# 4 bad guesses
"""
   +------+
   |      O
   |   ---+---
   |
   |   
   | 
   |
================
""",
# 5 bad guesses
"""
   +------+
   |      O
   |   ---+---
   |      |
   |   
   | 
   |
================
""",
# 6 bad guesses
"""
   +------+
   |      O
   |   ---+---
   |      |
   |     /   
   |    / 
   |
================
""",
# 7 bad guesses (game over)
"""
   +------+
   |      O
   |   ---+---
   |      |
   |     / \    
   |    /   \ 
   |
================
""")
    
    # Print out the text graphic figure based on the number of bad letter guesses.
    print(figures[bad_guesses])
    print()
                   
# Function to prompt the player to guess a letter in the word.
#
# The letter guess is read into a string. Any leading or trailing
# blanks are removed from the string and the result is converted
# to lowercase. The function returns the resulting string to
# the main program. 
#
def prompt_for_letter():
    print()
    # Assign the player's guess to the "answer" string variable.
    answer = str(input("Guess a letter in the word: "))
    # Remove any leading or trailing blanks from the guess.
    answer = answer.strip()
    # Convert the guess to all lowercase letters.
    answer = answer.lower()
    print()
    # Return what the player typed after stripping and converting to lowercase.
    return answer
   

###################################
# PROCESSING INITIALIZATION SECTION
###################################

#
# Define the tuple of words to be guessed. One of these words
# is chosen at random.
#
# (all words are animals)
# (tuples can't be changed)
#
animal_words = ("horse","cow","dog","cat","hyena","antelope","monkey","ox",\
               "moose","woodchuck","mouse","zebra")

# Select a random word from the animal_words list.
# choice() is a random function from the random module.
theword = choice(animal_words)

# Assign the length of the word chosen to the variable "word_len".
word_len = len(theword)

# Create a list to display letters not yet guessed and also
# those that have been guessed. There is one entry in this
# list for each letter in the mystery word. The list will contain
# either "_" if the letter has not yet been guessed, or a letter
# if it has been guessed. So "__a_t" would mean the letters "a" and "t"
# have been correctly guessed and that there are still three letters that
# have not yet been guessed by the player.
guesses = word_len*['_']

#
# Define variables used to keep track of the state
# of the game.
#

# Set the maximum number of incorrect letter guesses allowed.
max_incorrect = 7

# Define a string containing the letters already guessed.
# There are none to begin with.
letters_tried = ""

# Initialize count of letters guessed correctly.
letters_correct=0

# Initialize the count of incorrect letter guesses made so far.
incorrect_guesses=0

###################################
# PROCESSING SECTION
# Loop asking the player for a letter guess
# until they guess the word or are hanged for too
# many wrong guesses.
# Branching code: if/else
# Looping code: while-loop, for-loop
###################################

#
# Test the initialization code.
#
print("*** Test the initialization code ***")
print("animal_words",animal_words)
print("mystery word",theword)
print("word_len",word_len)
print("guesses",guesses)
print("letters_tried",letters_tried)

#
# Test all the defined functions.
#
print()
print("*** Testing the defined functions ***")
# Print the game rules and a clue about the mystery word to be guessed
# by the player.
print_game_rules(max_incorrect, word_len)

# Test the prompt_for_letter() function by calling it a few times
for i in range(3):
    a=prompt_for_letter()
    print("prompt_for_letter() returned <"+a+">")

# Test the display_figure() function
for i in range(8):
    print("bad guesses set to",i)
    display_figure(i)
    
###################################   
# CLEANUP, TERMINATION, AND EXIT SECTION
###################################

print("The game is over.")
