###################################
# PROLOG SECTION
# shuffle_cards.py
# Simulate shuffling and dealing from a deck of cards.
#
# Tested with Python 3.7.9 on Windows 10
# Fri July 16, 2021
# Copyright ©. All rights reserved. NAF 2021.
#
# Possible future enhancements:
# Unresolved bugs:
###################################

###################################
# ENVIRONMENT SETUP SECTION
# Imports: random
###################################
# import random number routines
from random import *

###################################
# FUNCTION DEFINITION SECTION
# Definitions: define_cards()
# create_deck(), shuffle_deck(),
# deal_cards()
###################################

#
# Function to define the cards in the deck.
#
# This function is nearly identical to define_cards.py,
# which should be used as a model.
# The function assigns string values to a list of
# integer variables representing the cards (52 in all).
# The list items range from 0 to 51.
# The value strings include both card rank
# (e.g., seven or queen) and suit (e.g., clubs or hearts).
# Examples: List item 0 represents string "ace of clubs";
# list item 2 represents "three of clubs";
# list item 51 represents "king of spades".
#
def define_cards(n):
    # strings for card ranks
    rank_string = ("ace","two","three","four","five","six","seven", \
                "eight","nine","ten","jack","queen","king")
    # strings for card suits
    suit_string = ("clubs","diamonds","hearts","spades")
    # initialize the list of cards to null
    cards = []
    # build up the strings for all cards in the deck
    # begin with the suits
    for suit in range(4):
        # first go through all ranks for spades, then clubs, etc.
        for rank in range(13):
            # assign the card string (e.g., "ace of clubs")
            # to rank_string
            card_string = rank_string[rank] + " of " + suit_string[suit]
            # append each card_string in turn
            # to the end of the list of cards
            cards.append(card_string)

    # return the entire list of cards to the calling function
    return cards[n]

#
# Function to create a deck.
#
def create_deck(deck):
    # deck was initialized to null in main program
    # create a deck that consists solely of 52 numbers
    # (0-51)
    # deck is sequenced like a new pack of cards
    # (i.e., rank-ordered within suit)
    # there is as yet no correspondence between the numbers
    # and the values established in define_cards
    for i in range(52):
        deck.append(i)
    # return to the calling function
    return

#
# Function to shuffle the deck.
#
# Call the random shuffle function to do the shuffle.
def shuffle_deck(deck):
    shuffle(deck)
    # return to the calling function
    return

#
# Function to "deal" a card.
#
# (in this case, simply turn a card over to reveal its face).
def deal_card(deck):    
    # return the first card in the deck to the calling function
    # hint: to deal from the bottom of the deck,
    # change the statement to: deck.pop()
    # hint: to see the effect, put in BOTH "print deck"
    # statements in the processing section 
    return deck.pop(0)

###################################
# PROCESSING INITIALIZATION SECTION
###################################
# initialize to an empty deck
# Hint: this statement must be outside the user-defined functions
# because it is used by multiple functions
deck=[]

###################################
# PROCESSING SECTION
# Shuffle the deck and deal the cards.
# Branching code: none
# Looping code: for-loops
###################################

# create a sequenced deck
# call the create_deck() user-defined function
create_deck(deck)

# shuffle the deck
# call the shuffle_deck() user-defined function
shuffle_deck(deck)

#
# deal 10 cards off the top of the shuffled deck
#
# Hint: the following 3 print statements are solely informational
# print informational message to the user
print()
print("This is the shuffled deck:")
# print the shuffled deck
print(deck)

# print informational message to the user
print()
print("These are the dealt cards:")

# loop to deal the first 10 cards
for i in range(10):
    # remove a card from the deck
    # call the deal_card() user-defined function
    card = deal_card(deck)
    # print the card definition (value)
    # call the define_cards() user-defined function
    print(define_cards(card))
    # Hint: if you have a "print deck" statement above,
    # and you want to see the "dealt" cards being removed
    # from the shuffled deck, one by one,
    # add another "print deck" statement here

###################################   
# CLEANUP, TERMINATION, AND EXIT SECTION
###################################
# print informational messages to the user
print()
print("End of program.")
print()
print("Run the program again to create a new deck \
and deal a new set of cards.")
